<?php

namespace Drupal\cloudconvert;

use Drupal\cloudconvert\Entity\CloudConvertTaskInterface;
use Drupal\cloudconvert\Entity\CloudConvertTaskTypeInterface;
use Drupal\file\FileInterface;

/**
 * Interface CloudConvertProcessorInterface
 *
 * @package Drupal\cloudconvert
 */
interface CloudConvertProcessorInterface {

  /**
   * Create the CloudConvert process.
   *
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskInterface $cloudConvertTask
   * @param \Drupal\cloudconvert\Parameters $parameters
   *
   * @return \Drupal\cloudconvert\Process
   */
  public function createProcess(CloudConvertTaskInterface $cloudConvertTask, Parameters $parameters);

  /**
   * Start the process for the given cloudconvert task.
   *
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskInterface $cloudConvertTask
   *
   * @return \Drupal\cloudconvert\Process
   */
  public function startProcess(CloudConvertTaskInterface $cloudConvertTask);

  /**
   * Finish the given cloudconvert task.
   *
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskInterface $cloudConvertTask
   */
  public function finishProcess(CloudConvertTaskInterface $cloudConvertTask);

  /**
   * Get the CloudConvert Process.
   *
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskInterface $cloudConvertTask
   *
   * @return \Drupal\cloudconvert\Process
   */
  public function getProcess(CloudConvertTaskInterface $cloudConvertTask);

  /**
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskTypeInterface $cloudConvertTaskType
   * @param \Drupal\file\FileInterface $file
   *
   * @return \Drupal\cloudconvert\Entity\CloudConvertTaskInterface
   */
  public function createTask(CloudConvertTaskTypeInterface $cloudConvertTaskType, FileInterface $file);

  /**
   * Download the file and create a file entity.
   *
   * @param \Drupal\cloudconvert\Process $process
   *
   * @return \Drupal\Core\Entity\EntityInterface
   */
  public function downloadFile(Process $process);

  /**
   * Get the info data from the process.
   *
   * @param \Drupal\cloudconvert\Process $process
   *
   * @return object
   */
  public function gatherInfo(Process $process);

  /**
   * Get the CloudConvert API.
   *
   * @return \Drupal\cloudconvert\Api
   */
  public function getCloudConvertApi();

  /**
   * Create a a Queue Item to start a process.
   *
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskInterface $cloudConvertTask
   * @param \Drupal\cloudconvert\Parameters $parameters
   */
  public function createStartQueueItem(CloudConvertTaskInterface $cloudConvertTask, Parameters $parameters);

  /**
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskInterface $cloudConvertTask
   */
  public function createFinishQueueItem(CloudConvertTaskInterface $cloudConvertTask);

  /**
   * Get the HOOK url for CloudConvert to call when the task is finished.
   *
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskInterface $cloudConvertTask
   *
   * @return \Drupal\Core\Url
   */
  public function getCallbackUrl(CloudConvertTaskInterface $cloudConvertTask);

}
